package cn.com.qmth.examcloud.core.basic.service.impl;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.transaction.Transactional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.constants.Consts;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.dao.entity.Student;
import cn.com.qmth.examcloud.core.basic.service.StudentService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentInfo;

/**
 * 学生服务类 Created by songyue on 17/1/14.
 */
@Service
public class StudentServiceImpl implements StudentService {

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	UserRepo userRepo;

	@Autowired
	UserServiceImpl userService;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	DataSendService dataSendService;

	@Autowired
	OrgService orgService;

	private static final String JPG = ".jpg";

	/**
	 * 获取所有学生(分页)
	 *
	 * @param studentCriteria
	 * @param pageable
	 * @return
	 */
	public Page<Student> getAllStudent(Student studentCriteria, Pageable pageable) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains())
				.withMatcher("studentCode", contains()).withMatcher("identityNumber", contains())
				.withIgnoreNullValues();
		Example<Student> studentExample = Example.of(studentCriteria, exampleMatcher);
		return studentRepo.findAll(studentExample, pageable);
	}

	/**
	 * 获取所有学生
	 *
	 * @param studentCriteria
	 * @return
	 */
	public List<Student> getAllStudent(Student studentCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching();
		Example<Student> studentExample = Example.of(studentCriteria, exampleMatcher);
		return studentRepo.findAll(studentExample);
	}

	/**
	 * 绑定学生照片
	 *
	 * @return
	 */
	public List<ErrorMsg> photopBundled(Long rootOrgId, File directory) {
		List<ErrorMsg> list = new ArrayList<ErrorMsg>();
		if (directory.isDirectory()) {
			for (File file : directory.listFiles()) {
				if (!file.getName().endsWith(JPG)) {
					list.add(new ErrorMsg(file.getName() + "格式错误，文件名格式必须是jpg(小写)"));
				}
				String fileName = file.getName().split(JPG)[0];
				// 先查学号
				Student student = studentRepo.findByStudentCodeAndRootOrgId(fileName, rootOrgId);
				if (student == null) {// 学号不存在查身份证号
					student = studentRepo.findByIdentityNumber(fileName);
				}
				if (student != null) {// 学生存在则绑定照片
					student.setPhotoPath(student.getRootOrgId() + File.separator + "photo"
							+ File.separator + fileName);
					studentRepo.save(student);
					dataSendService.sendStudent(student);
				}
			}
		} else {
			list.add(new ErrorMsg("文件路径不正确"));
		}
		return list;
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see cn.com.qmth.examcloud.core.basic.service.StudentService#
	 * insertOrUpdateStudent(cn.com.qmth.examcloud.core.basic.service.bean.
	 * StudentInfo)
	 */
	@Override
	@Transactional
	public Student insertOrUpdateStudent(StudentInfo studentInfo) {
		Long rootOrgId = studentInfo.getRootOrgId();
		Org rootOrg = orgRepo.findOne(rootOrgId);

		if (null == rootOrg || null != rootOrg.getParentId()) {
			throw new StatusException("B-160001", "顶级机构错误");
		}

		Long orgId = studentInfo.getOrgId();
		String orgCode = studentInfo.getOrgCode();
		String orgName = studentInfo.getOrgName();

		if (null != orgId) {
			Org org = orgRepo.findOne(orgId);
			if (!org.getParentId().equals(rootOrgId)) {
				throw new StatusException("B-160002", "机构错误");
			}
		} else {
			if (StringUtils.isBlank(orgName)) {
				throw new StatusException("B-160003", "orgName is blank");
			}
			if (StringUtils.isBlank(orgCode)) {
				throw new StatusException("B-160004", "orgCode is blank");
			}
			Org org = orgRepo.findByRootIdAndCode(rootOrgId, orgCode);
			if (null == org) {
				org = new Org();
				org.setParentId(rootOrgId);
				org.setCode(orgCode);
				org.setName(orgName);
				Org saved = orgRepo.save(org);
				orgId = saved.getId();
			}
		}

		String identityNumber = studentInfo.getIdentityNumber();
		if (StringUtils.isBlank(identityNumber)) {
			throw new StatusException("B-160012", "身份证号不能为空");
		}
		String studentCode = studentInfo.getStudentCode();
		if (StringUtils.isBlank(studentCode)) {
			studentCode = null;
		}

		List<Student> list = studentRepo.findAllByIdentityNumberAndRootOrgId(identityNumber,
				rootOrgId);
		if (1 < list.size()) {
			throw new StatusException("B-160007", "已经存在不同考生的身份证号相同的数据. 身份证号: " + identityNumber);
		}
		if (StringUtils.isNotBlank(studentCode)) {
			list = studentRepo.findAllByStudentCodeAndRootOrgId(studentCode, rootOrgId);
			if (1 < list.size()) {
				throw new StatusException("B-160008", "已经存在不同考生的学号相同的数据. 学号: " + studentCode);
			}
		}

		Student studentByIdentity = studentRepo.findByIdentityNumberAndRootOrgId(identityNumber,
				rootOrgId);

		if (null != studentByIdentity) {
			String curStudentCode = studentByIdentity.getStudentCode();
			if (StringUtils.isNotBlank(curStudentCode) && !curStudentCode.equals(studentCode)) {
				throw new StatusException("B-160005", "身份证号已关联学号");
			}
		}

		if (StringUtils.isNotBlank(studentCode)) {
			Student studentByCode = studentRepo.findByStudentCodeAndRootOrgId(studentCode,
					rootOrgId);
			if (null != studentByCode) {
				String curIdentityNumber = studentByCode.getIdentityNumber();
				if (!curIdentityNumber.equals(identityNumber)) {
					throw new StatusException("B-160006", "学号已关联身份证号");
				}
			}
		}

		Student student = null;
		if (null != studentByIdentity) {
			student = studentByIdentity;
			if (null == student.getEnable()) {
				student.setEnable(true);
			}
		} else {
			student = new Student();
			if (StringUtils.isNotEmpty(identityNumber)
					&& identityNumber.matches("[0-9a-zA-Z]{6,}")) {
				student.setPassword(
						StringUtils.substring(identityNumber, -6, identityNumber.length()));
			} else {
				student.setPassword(Consts.DEFAULT_PASSWORD);
			}
		}

		if (null == student.getEnable()) {
			student.setEnable(true);
		}
		student.setRootOrgId(rootOrgId);
		student.setIdentityNumber(identityNumber);
		student.setStudentCode(studentCode);
		student.setOrgId(orgId);
		if (null != studentInfo.getName()) {
			student.setName(studentInfo.getName());
		}
		if (null != studentInfo.getPhotoPath()) {
			student.setPhotoPath(studentInfo.getPhotoPath());
		}
		if (null != studentInfo.getPhoneNumber()) {
			student.setPhoneNumber(studentInfo.getPhoneNumber());
		}
		if (null != studentInfo.getRemark()) {
			student.setRemark(studentInfo.getRemark());
		}
		if (null != studentInfo.getEnable()) {
			student.setEnable(studentInfo.getEnable());
		}
		Student saved = studentRepo.save(student);

		return saved;
	}

}
